@@ -44,20 +44,19 @@ struct _GnomePrintPreviewPrivate {
 	int top_page;
 	int current_page;
 
-	/* The root group, with a translation setup */
-	GnomeCanvasItem *root;
-
 	/* The current page */
 	GnomeCanvasItem *page;
 
 	/* Strict theme compliance [#96802] */
-	gboolean theme_compliance; 
+	gboolean theme_compliance;
+
+	gboolean use_theme;
 };
 
 static GnomePrintContextClass *parent_class;
 
 static void
-outline_set_style_cb (GtkWidget *canvas, GtkStyle *ps, GnomeCanvasItem *item)
+outline_set_style_cb (GtkWidget *canvas, GnomeCanvasItem *item)
 {
 	gint32 color;
 	GtkStyle *style;
@@ -102,22 +101,19 @@ gpp_stroke (GnomePrintContext *pc, const ArtBpath *bpath)
 
 	gnome_canvas_path_def_unref (path);
 
-	if (pp->priv->theme_compliance) {
-		g_signal_connect (G_OBJECT (item->canvas), "style_set",
-				  G_CALLBACK (outline_set_style_cb), item);
-	}
-	
+	if (pp->priv->use_theme)
+		outline_set_style_cb (GTK_WIDGET (item->canvas), item);	
 	return 1;
 }
 
 static void
-fill_set_style_cb (GtkWidget *canvas, GtkStyle *ps, GnomeCanvasItem *item)
+fill_set_style_cb (GtkWidget *canvas, GnomeCanvasItem *item)
 {
 	gint32 color;
 	GtkStyle *style;
 
 	style = gtk_widget_get_style (GTK_WIDGET (canvas));
-	color = GPP_COLOR_RGBA (style->text [GTK_STATE_NORMAL], 0xff);
+	color = GPP_COLOR_RGBA (style->bg [GTK_STATE_NORMAL], 0xff);
 	
 	gnome_canvas_item_set (item, "fill_color_rgba", color, NULL);
 }
@@ -149,11 +145,8 @@ gpp_fill (GnomePrintContext *pc, const ArtBpath *bpath, ArtWindRule rule)
 		NULL);
 	gnome_canvas_path_def_unref (path);
 
-	if (pp->priv->theme_compliance) {
-		g_signal_connect (G_OBJECT (item->canvas), "style_set",
-				  G_CALLBACK (fill_set_style_cb), item);
-	}
-
+	if (pp->priv->use_theme)
+		fill_set_style_cb (GTK_WIDGET (item->canvas), item);
 	return 1;
 }
 
@@ -289,11 +282,22 @@ gpp_showpage (GnomePrintContext *pc)
 static int
 gpp_beginpage (GnomePrintContext *pc, const guchar *name)
 {
+	GnomePrintPreview *preview = GNOME_PRINT_PREVIEW (pc);
+	GnomeCanvasGroup *group = GNOME_CANVAS_GROUP (preview->priv->page);
+
+	while (group->item_list) {
+		GnomeCanvasItem *child = group->item_list->data;
+
+		group->item_list = g_list_delete_link (group->item_list,
+				group->item_list);
+		gtk_object_destroy (GTK_OBJECT (child));
+	}
+
 	return GNOME_PRINT_OK;
 }
 
 static void
-glyphlist_set_style_cb (GtkWidget *canvas, GtkStyle *ps, GnomeCanvasItem *item)
+glyphlist_set_style_cb (GtkWidget *canvas, GnomeCanvasItem *item)
 {
 	GnomeGlyphList *gl, *new;
 	gint32 color;
@@ -340,11 +344,8 @@ gpp_glyphlist (GnomePrintContext *pc, const gdouble *affine, GnomeGlyphList * gl
 
 	gnome_canvas_item_affine_absolute (item, transform);
 
-	if (pp->priv->theme_compliance) {
-		g_signal_connect (G_OBJECT (item->canvas), "style_set",
-				  G_CALLBACK (glyphlist_set_style_cb), item);
-	}
-	
+	if (pp->priv->use_theme)
+		glyphlist_set_style_cb (GTK_WIDGET (item->canvas), item);	
 	return 0;
 }
 
@@ -386,20 +387,82 @@ gpp_finalize (GObject *object)
 
 	gp_gc_unref (priv->gc);
 
-	if (pp->canvas)
-		g_object_unref (G_OBJECT (pp->canvas));
-
 	if (priv->page)
 		gtk_object_destroy (GTK_OBJECT (priv->page));
 
-	if (priv->root)
-		gtk_object_destroy (GTK_OBJECT (priv->root));
-	
 	g_free (priv);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+enum {
+	PROP_0,
+	PROP_GROUP,
+	PROP_USE_THEME,
+	PROP_THEME_COMPLIANCE
+};
+
+static void
+gpp_get_property (GObject *gobject, guint param_id,
+		  GValue *value, GParamSpec *pspec)
+{
+	GnomePrintPreview *preview = GNOME_PRINT_PREVIEW (gobject);
+	
+	switch (param_id) {
+	case PROP_GROUP:
+		g_value_set_object (value, preview->priv->page);
+		break;
+	case PROP_THEME_COMPLIANCE:
+		g_value_set_boolean (value, preview->priv->theme_compliance);
+		break;
+	case PROP_USE_THEME:
+		g_value_set_boolean (value, preview->priv->use_theme);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, param_id, pspec);
+	}
+}
+
+static void
+gnome_print_preview_construct (GnomePrintPreview *preview,
+			       GnomeCanvasGroup *group)
+{
+	GnomePrintConfig *config;
+
+	g_return_if_fail (GNOME_IS_PRINT_PREVIEW (preview));
+	g_return_if_fail (GNOME_IS_CANVAS_GROUP (group));
+	g_return_if_fail (!preview->priv->page);
+
+	config = gnome_print_config_default ();
+	gnome_print_context_construct (GNOME_PRINT_CONTEXT (preview), config);
+	g_object_unref (config);
+
+	preview->priv->page = GNOME_CANVAS_ITEM (group);
+	gp_gc_set_data (preview->priv->gc, group);
+}
+
+static void
+gpp_set_property (GObject *gobject, guint param_id,
+		  const GValue *value, GParamSpec *pspec)
+{
+	GnomePrintPreview *preview = GNOME_PRINT_PREVIEW (gobject);
+
+	switch (param_id) {
+	case PROP_GROUP:
+		gnome_print_preview_construct (preview,
+			GNOME_CANVAS_GROUP (g_value_get_object (value)));
+		break;
+	case PROP_THEME_COMPLIANCE:
+		preview->priv->theme_compliance = g_value_get_boolean (value);
+		break;
+	case PROP_USE_THEME:
+		preview->priv->use_theme = g_value_get_boolean (value);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, param_id, pspec);
+	}
+}
+
 static void
 class_init (GnomePrintPreviewClass *klass)
 {
@@ -408,8 +471,7 @@ class_init (GnomePrintPreviewClass *klass)
 
 	parent_class = g_type_class_peek_parent (klass);
 
-	object_class->finalize = gpp_finalize;
-	
+	/* Set up the print context class */
 	pc_class->beginpage = gpp_beginpage;
 	pc_class->showpage  = gpp_showpage;
 	pc_class->clip      = gpp_clip;
@@ -420,6 +482,23 @@ class_init (GnomePrintPreviewClass *klass)
 	pc_class->gsave     = gpp_gsave;
 	pc_class->grestore  = gpp_grestore;
 	pc_class->close     = gpp_close;
+
+	/* Set upt the object class */
+	object_class->finalize = gpp_finalize;
+	object_class->get_property = gpp_get_property;
+	object_class->set_property = gpp_set_property;
+	g_object_class_install_property (G_OBJECT_CLASS (object_class),
+		PROP_GROUP, g_param_spec_object ("group",
+			"Group", "Group", GNOME_TYPE_CANVAS_GROUP,
+			G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_install_property (G_OBJECT_CLASS (object_class),
+		PROP_THEME_COMPLIANCE, g_param_spec_boolean ("theme_compliance",
+			"Theme compliance", "Theme compliance", TRUE,
+			G_PARAM_READWRITE));
+	g_object_class_install_property (G_OBJECT_CLASS (object_class),
+		PROP_USE_THEME, g_param_spec_boolean ("use_theme",
+			"Use theme", "Use theme", FALSE,
+			G_PARAM_READWRITE));
 }
 
 static void
@@ -436,21 +515,6 @@ clear_val (GtkObject *object, void **val)
 	*val = NULL;
 }
 
-/**
- * gnome_print_preview_theme_compliance:
- * @pp: 
- * @compliance: 
- * 
- * This has to go away for GNOME 2.4. Make compliance an argument of
- * gnome_print_preview_new_full
- **/
-void
-gnome_print_preview_theme_compliance (GnomePrintPreview *pp,
-				      gboolean compliance)
-{
-	pp->priv->theme_compliance = compliance;
-}
-
 GnomePrintContext *
 gnome_print_preview_new_full (GnomePrintConfig *config, GnomeCanvas *canvas,
 			      const gdouble *transform, const ArtDRect *region)
@@ -475,23 +539,14 @@ gnome_print_preview_new_full (GnomePrintConfig *config, GnomeCanvas *canvas,
 	}
 	preview = GNOME_PRINT_PREVIEW (ctx);
 
-	g_object_ref (G_OBJECT (canvas));
-	preview->canvas = canvas;
-
 	gnome_canvas_set_scroll_region (canvas, region->x0, region->y0, region->x1, region->y1);
 
-	preview->priv->root = gnome_canvas_item_new (gnome_canvas_root (preview->canvas),
-						     GNOME_TYPE_CANVAS_GROUP,
-						     "x", 0.0, "y", 0.0, NULL);
-
-	preview->priv->page = gnome_canvas_item_new (gnome_canvas_root (preview->canvas),
+	preview->priv->page = gnome_canvas_item_new (gnome_canvas_root (canvas),
 						     GNOME_TYPE_CANVAS_GROUP,
 						     "x", 0.0, "y", 0.0, NULL);
 
 	g_signal_connect (G_OBJECT (preview->priv->page), "destroy",
 			  (GCallback) clear_val, &preview->priv->page);
-	g_signal_connect (G_OBJECT (preview->priv->root), "destroy",
-			  (GCallback) clear_val, &preview->priv->root);
 
 	/* Setup base group */
 	group = GNOME_CANVAS_GROUP (preview->priv->page);
