@@ -48,6 +48,11 @@ static void gpa_printer_selector_sync_printer (GtkListStore *model,
 					       GPAPrinter   *printer);
 static void selection_changed_cb (GtkTreeSelection *selection, 
 				  gpointer          data);
+gboolean interactive_search_func (GtkTreeModel *model,
+				  gint          column,
+				  const gchar  *key,
+				  GtkTreeIter  *iter,
+				  gpointer search_data);
 static void set_printer_icon (GtkCellLayout   *layout, 
 			      GtkCellRenderer *rend,
 			      GtkTreeModel    *model, 
@@ -163,6 +168,11 @@ gpa_printer_selector_init (GPAPrinterSelector *ps)
 						 (GtkTreeCellDataFunc)set_printer_name, 
 						 NULL, NULL);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (ps->treeview), col);
+	gtk_tree_view_set_search_column (GTK_TREE_VIEW (ps->treeview), 0);
+	gtk_tree_view_set_search_equal_func (GTK_TREE_VIEW (ps->treeview),
+					     interactive_search_func,
+					     NULL,
+					     NULL);
 
 	rend = gtk_cell_renderer_text_new ();
 	col = gtk_tree_view_column_new_with_attributes (_("State"), rend, NULL);	
@@ -263,6 +273,26 @@ printer_sort_func (GtkTreeModel *model,
 	}
 }
 
+gboolean
+interactive_search_func (GtkTreeModel *model,
+			 gint column,
+			 const gchar *key,
+			 GtkTreeIter *iter,
+			 gpointer search_data)
+{
+	char *name;
+	GPANode *printer;
+	gboolean ret;
+
+	gtk_tree_model_get (model, iter, 0, &printer, -1);	
+	name = gpa_node_get_value (printer);
+	
+	ret = strncmp (key, name, strlen (key));
+	g_free (name);
+
+	return ret;
+}
+
 void
 gpa_printer_selector_printer_state_changed (GPAPrinterSelector *selector,
 					    GPANode *printer)
@@ -282,7 +312,7 @@ selection_changed_cb (GtkTreeSelection *selection, gpointer data)
 	GPAPrinterSelector *selector;
 	GtkTreeIter sort_iter;
 	GtkTreeIter iter;
-	GPANode *node;
+	GPANode *node, *printer;
 
 	selector = GPA_PRINTER_SELECTOR (data);
 	if (!gtk_tree_selection_get_selected (selection, NULL, &sort_iter))
@@ -291,7 +321,22 @@ selection_changed_cb (GtkTreeSelection *selection, gpointer data)
 							&iter, &sort_iter);
 	gtk_tree_model_get (selector->model, &iter, 0, &node, -1);
 
-	gpa_node_set_path_value (selector->config, "Printer", gpa_node_id (node));
+	/* We used to do just 
+         * gpa_node_set_path_value (selector->config, "Printer", gpa_node_id (node))
+         * but that doesn't work due to a couple of interacting peculiarities of
+	 * gpa lists and references:
+	 * a) default list members are not reset when the node is removed from the list,
+         *    the parent of the removed node is reset to NULL, though
+         * b) for reference nodes, gpa_node_set_path_value() uses the parent of the old
+         *    value to lookup the new value by id
+         * c) gpa_reference_set_reference() doesn't emit the ::modified signal
+         */
+	printer = gpa_node_lookup (selector->config, "Printer");
+	gpa_reference_set_reference (GPA_REFERENCE (printer), node);
+	gpa_node_emit_modified (printer);
+	
+	if (!selector->changing_default)
+		selector->user_selected = TRUE;
 }
 
 /**
@@ -438,20 +483,56 @@ gpa_printer_selector_sync_printer (GtkListStore *model,
 	gtk_list_store_set (GTK_LIST_STORE (model), iter, 0, printer, -1);
 }
 
+static gboolean
+scroll_to_selected (GPAPrinterSelector *ps)
+{
+	GtkTreeIter iter;
+	GtkTreePath *path;	
+	GtkTreeSelection *selection;
+
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (ps->treeview));
+	if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
+		path = gtk_tree_model_get_path (ps->sortmodel, &iter);
+		gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (ps->treeview),
+					      path, NULL, FALSE, 0.0, 0.0);
+		gtk_tree_path_free (path);
+	}
+	
+	return FALSE;
+}
+
 static void
 gpa_printer_selector_printer_added_cb (GPANode *parent, GPANode *child,
 				       GPAPrinterSelector *ps)
 {
-	GtkTreeIter iter;
+ 	GtkTreeIter iter, sort_iter;
+ 	GtkTreeSelection *selection;
+ 	GPANode *default_printer = NULL;
 
 	GDK_THREADS_ENTER ();
 
 	g_return_if_fail (node_to_iter (ps->model, child, &iter) == FALSE);
 
 	gtk_list_store_append (GTK_LIST_STORE (ps->model), &iter);
+
 	gpa_printer_selector_sync_printer (GTK_LIST_STORE (ps->model), &iter,
 					   GPA_PRINTER (child));
 
+  	if (!ps->user_selected && ps->config != NULL)
+  		default_printer = GPA_REFERENCE_REFERENCE (GPA_CONFIG (ps->config)->printer);
+  
+  	if (default_printer != NULL &&
+  	    node_to_iter (ps->model, default_printer, &iter)) {
+  		gtk_tree_model_sort_convert_child_iter_to_iter (GTK_TREE_MODEL_SORT (ps->sortmodel), 
+ 								&sort_iter, &iter);
+  		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (ps->treeview));
+ 		ps->changing_default = TRUE;
+  		gtk_tree_selection_select_iter (selection, &sort_iter);
+ 		ps->changing_default = FALSE;
+ 	}
+ 
+ 	scroll_to_selected (ps);
+
 	GDK_THREADS_LEAVE ();
 }
 
@@ -475,10 +556,6 @@ gpa_printer_selector_construct (GPAWidget *gpa)
 {
 	GPAPrinterSelector *ps;
 	GPANode *child;
-	GtkTreeIter iter;
-	GtkTreeIter sort_iter;
-	GPANode *default_printer = NULL;
-	GtkTreeSelection *selection;
 
 	ps = GPA_PRINTER_SELECTOR (gpa);
 	ps->config = GNOME_PRINT_CONFIG_NODE (gpa->config);
@@ -492,25 +569,10 @@ gpa_printer_selector_construct (GPAWidget *gpa)
 
 	child = gpa_node_get_child (ps->node, NULL);
 	while (child) {
-		GtkTreeIter iter;
-		gtk_list_store_append (GTK_LIST_STORE (ps->model), &iter);
-		gpa_printer_selector_sync_printer (GTK_LIST_STORE (ps->model),
-						   &iter,
-						   GPA_PRINTER (child));
+  		gpa_printer_selector_printer_added_cb (ps->node, child, ps);
 		child = gpa_node_get_child (ps->node, child);
 	}
 
-	if (ps->config != NULL)
-		default_printer = GPA_REFERENCE_REFERENCE (GPA_CONFIG (ps->config)->printer);
-
-	if (default_printer != NULL &&
-	    node_to_iter (ps->model, default_printer, &iter), FALSE) {
-		gtk_tree_model_sort_convert_child_iter_to_iter (
-			GTK_TREE_MODEL_SORT (ps->sortmodel), &sort_iter, &iter);
-		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (ps->treeview));
-		gtk_tree_selection_select_iter (selection, &sort_iter);
-	}
-
 	return TRUE;
 }
 
